<?php

/*
CLASSE COPIADA COM MODIFICAÇÕES DA DEPENDENCIA 'TEKNISA/LIBRARIES' VERSÃO: '3.0.24'. NAMESPACE ORIGINAL 'Teknisa\Libs\Controller'
*/

namespace Ems\BancoTalentos\Auth\Controller;

use Ems\BancoTalentos\Auth\InstanceProvider;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

use Ems\BancoTalentos\Auth\Service\Login as LoginService;
use Ems\BancoTalentos\Auth\Service\Api as ApiService;
use Ems\BancoTalentos\Auth\Util\Utilities as LibUtilities;
use Ems\BancoTalentos\Util\Utilities;
use Ems\BancoTalentos\Auth\RequestListenerOAuth;
use Ems\BancoTalentos\Util\RequestUtils;
use Ems\BancoTalentos\Util\Messages;

use GuzzleHttp\Client;

class Login {
    /** @var LoginService $loginService */
    private $loginService;
    /** @var ApiService $apiService */
    private $apiService;

    /**
     * @param LoginService $loginService
     * @param ApiService $apiService
     */

    //  mudar paths para um arquivo expecifico
    public function __construct(LoginService $loginService, ApiService $apiService, RequestListenerOAuth $requestListenerOAuth, RequestUtils $requestUtils) {
        $this->loginService         = $loginService;
        $this->apiService           = $apiService;
        $this->requestListenerOAuth = $requestListenerOAuth;
        $this->requestUtils         = $requestUtils;
    }

    const DEFAULT_PROJECT       = '3914';
    const DEFAULT_ERASE_HASH    = false;
    const DEFAULT_HASH          = null;

    /**
     * @param Request\Filter $request
     * @param Response $response
     * @throws \Exception
     */
    // public function startSession(Request\Row $request, Response $response) {
    public function startSession (
        string  $loginUser,
        string  $loginToken,
        bool    $isLogged = true
    ) {
        try {
            $modulesVersionError = null;

            $loginUserInfo = @json_decode(LibUtilities::zeedhiDecrypt($loginUser), true);
            if(empty($loginUserInfo)) {
                $loginUserInfo = InstanceProvider::getSession()->get('LOGIN_USER_INFO');
            } else {
                unset($loginUserInfo['CURRENT_SECONDS']);
            }
            if(isset($loginUserInfo['USER_DATA']) && !empty($loginUserInfo['USER_DATA'])) {
                $userData = $loginUserInfo['USER_DATA'];
            } else {
                $userData = $this->apiService->getUserData($loginUserInfo['USER_EMAIL'], null);
            }
            $userData = $this->loginService->restoreSession($userData, $loginUserInfo, $isLogged, $modulesVersionError, $loginToken);

            return $userData;
        } catch(\Exception $e) {
            return null;
        }
    }

    public function startSessionRequest(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $isLogged       = $row['IS_LOGGED'];
            $loginUser      = $row['USER'];
            $loginToken     = $row['LOGIN_TOKEN'];

            $this->requestListenerOAuth->checkAccessAndSetSession($loginToken);

            $modulesVersionError = null;

            $loginUserInfo = @json_decode(LibUtilities::zeedhiDecrypt($loginUser), true);
            if(empty($loginUserInfo)) {
                $loginUserInfo = InstanceProvider::getSession()->get('LOGIN_USER_INFO');
            } else {
                unset($loginUserInfo['CURRENT_SECONDS']);
            }
            if(isset($loginUserInfo['USER_DATA']) && !empty($loginUserInfo['USER_DATA'])) {
                $userData = $loginUserInfo['USER_DATA'];
            } else {
                $userData = $this->apiService->getUserData($loginUserInfo['USER_EMAIL'], null);
            }
            $userData = $this->loginService->restoreSession($userData, $loginUserInfo, $isLogged, $modulesVersionError, $loginToken);

            $response->addDataSet(new DataSet('system', $userData));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    // ATUALMENTE NÃO ESTÁ EM UTILIZAÇÃO. O FRONT ESTÁ BUSCANDO DIRETAMENTE PELO MÓDULO DE LOGIN E INICIANDO A SESSÃO PELO 'startSessionRequest' LOGO APÓS
    // Rotina de login que inicia a sessão do operador
    public function login(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $email          = $row['EMAIL'];
            $password       = $row['PASSWORD'];

            // separar o product_id e o requester
            $productId      = Utilities::getProductId();
            $requesterUrl   = $row['REQUESTER_URL'];
            $attempts       = $row['ATTEMPTS'];

            // Faz requisição para o módulo de login para validar o acesso
            $loginReponse = $this->requestUtils->makeLoginRequest(
                $email,
                $password,
                $productId,
                $requesterUrl,
                $attempts
            );

            // Validações sobre a resposta
            $userData = $this->getLoginData($loginReponse);

            $loginUser  = $userData['USER'];
            $loginToken = $userData['TOKEN'];
            $isLogged   = $userData['LOGGED'];

            // Inicia a sessão
            $this->requestListenerOAuth->checkAccessAndSetSession($loginToken);

            $sessionData = $this->startSession(
                $loginUser,
                $loginToken
            );
            
            if (is_null($sessionData)) {
                throw new \Exception(Messages::SESS_FAILED_TO_START);
            }

            $loginData = array_merge($userData, $sessionData);

            $response->addDataSet(new DataSet('system', $loginData));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    // Rotina de logout que interrompe a sessão do operador
    public function logout(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $project    = isset($row['PROJECT'])    ? $row['PROJECT']       : self::DEFAULT_PROJECT;
            $eraseHash  = isset($row['ERASE_HASH']) ? $row['ERASE_HASH']    : self::DEFAULT_ERASE_HASH;
            $hash       = isset($row['HASH'])       ? $row['HASH']          : self::DEFAULT_HASH;

            $this->loginService->logout(
                $project,
                $eraseHash,
                $hash
            );

            $response->addDataSet(new DataSet('system', array('success' => true)));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function loginValidations($loginReponse) {
        if (isset($loginReponse['error'])) {
            throw new \Exception(Messages::AUTH_PASSWORD_INCORRECT);
        }

        if (!isset($loginReponse['dataset']) || !isset($loginReponse['dataset']['userData'])) {
            throw new \Exception(Messages::AUTH_UNESPECTED_ERROR);
        }

        $userData = $loginReponse['dataset']['userData'];
        if (!$userData['LOGGED']) {
            throw new \Exception(Messages::AUTH_LOGIN_FAILED);
        }

        return $userData;
    }

    public function getLoginData($loginReponse) {
        $userData = $this->loginValidations($loginReponse);
        return $userData;
    }
}