<?php

/*
CLASSE COPIADA COM MODIFICAÇÕES DA DEPENDENCIA 'TEKNISA/LIBRARIES' VERSÃO: '3.0.24'. NAMESPACE ORIGINAL 'Teknisa\Libs\Service'
*/

namespace Ems\BancoTalentos\Auth\Service;

use Zeedhi\Framework\Session\Session;

use Ems\BancoTalentos\Auth\InstanceProvider;
use Ems\BancoTalentos\Auth\Util\Utilities;
use Ems\BancoTalentos\Auth\Util\Environment;
use Ems\BancoTalentos\Auth\Repositories;

class Login {
    /** @var Session $sessionManager */
    protected $sessionManager;
    protected $loginUrl;
    protected $loginClientSecret;

    /** @var Utilities */
    private $utilities;
    /** @var Environment */
    private $environment;
    /** @var ActiveDirectory */
    private $activeDirectory;
    /** @var Repositories\UserData */
    private $userDataRepository;

    /**
     * @param $sessionManager
     * @param Utilities $utilities
     * @param Environment $environment
     * @param ActiveDirectory $activeDirectory
     * @param Repositories\UserData $userDataRepository
     */
    public function __construct($sessionManager, Repositories\UserData $userDataRepository, Environment $environment, Utilities $utilities){
        $this->sessionManager       = $sessionManager;
        $this->userDataRepository   = $userDataRepository;
        $this->environment          = $environment;
        $this->utilities            = $utilities;
    }

    /**
     * @param $userData
     * @param $loginUserInfo
     * @param $isLogged
     * @param $modulesVersionError
     * @param $loginToken
     * @throws \Exception
     * @return array
     */
    public function restoreSession($userData, $loginUserInfo, $isLogged, $modulesVersionError, $loginToken) {
        if(!empty($userData)) {
            if(isset($loginUserInfo['CURRENT_SECONDS'])) {
                if (microtime(true) - $loginUserInfo['CURRENT_SECONDS'] > 60) {
                    return array();
                } else {
                    unset($loginUserInfo['CURRENT_SECONDS']);
                }
            }
            if(empty($userData['USER'])) {
                return array();
            }
            $loginUser          = Utilities::zeedhiEncrypt(json_encode($loginUserInfo));
            $user               = $userData['USER'];
            $organization       = $userData['ORGANIZATION'];
            $accessProfile      = Utilities::getArrayFromAssociativeArray($userData['ACCESS_PROFILE']);
            $otherSessionValues = isset($userData['OTHER_SESSION_VALUES']) ? $userData['OTHER_SESSION_VALUES'] : array();
            
            $this->initSession($user, $organization, $accessProfile, $otherSessionValues, $loginUserInfo);
            $userData = $this->factorySessionData($user, $organization, $accessProfile, $otherSessionValues, $loginUserInfo,
                $modulesVersionError, $loginUser, $loginToken);

            if(!$isLogged) {
                $entityManager = InstanceProvider::getConnectionCustom()->updateVpdConnection($userData['ORGANIZATION_ID']);
                if($entityManager) {
                    $this->userDataRepository->setEntityManager($entityManager);
                }
                $this->userDataRepository->updateUserLastAccess($userData['USER_ID'], $userData['ORGANIZATION_ID']);
            }

            return $userData;
        } else {
            throw new CustomException('ERR_REQUEST_USER_DATA');
        }
    }

    /**
     * @param $user
     * @param $organization
     * @param $accessProfile
     * @param $otherSessionValues
     * @param $loginUserInfo
     */
    private function initSession($user, $organization, $accessProfile, $otherSessionValues, $loginUserInfo)
    {
        if(!isset($loginUserInfo['CURRENT_SECONDS'])) {
            $loginUserInfo['CURRENT_SECONDS'] = microtime(true);
        }
        
        $idUtlPerfilFilial = $this->userDataRepository->getIdUtlPerfilFilial($user['ORGANIZATION_ID']);
        if($idUtlPerfilFilial['IDUTLPERFILFILIAL'] == 'S') {
            $perfilSelecionado = $this->userDataRepository->getPerfilSelecionado($user['ORGANIZATION_ID'], $user['ID']);
            $this->sessionManager->set('CDPERFILSELECIONADO', $perfilSelecionado['CDPERFILSELECIONADO']);
        }

        $this->sessionManager->start();

        $this->sessionManager->set('USER_ID',         $user['ID']);
        $this->sessionManager->set('USER_NAME',       $user['NAME']);
        $this->sessionManager->set('EMAIL',           $user['EMAIL']);
        $this->sessionManager->set('ORGANIZATION_ID', $user['ORGANIZATION_ID']);
        $this->sessionManager->set('TRADING_NAME',    $organization['TRADING_NAME']);

        $this->sessionManager->set('CDOPERADOR',  $user['ID']);
        $this->sessionManager->set('NMOPERADOR',  $user['NAME']);
        $this->sessionManager->set('DSEMAILOPER', $user['EMAIL']);
        $this->sessionManager->set('NRORG',       $user['ORGANIZATION_ID']);
        $this->sessionManager->set('NMORG',       $organization['TRADING_NAME']);
        
        $this->sessionManager->set('IDUTLPERFILFILIAL', $idUtlPerfilFilial['IDUTLPERFILFILIAL']);

        if(isset($loginUserInfo['FILE_SERVER_API_KEY']))
            $this->sessionManager->set('FILE_SERVER_API_KEY', $loginUserInfo['FILE_SERVER_API_KEY']);
            
        $this->sessionManager->set('ACCESS_PROFILE',  $accessProfile);
        $this->sessionManager->set('LOGIN_USER_INFO',  $loginUserInfo);

        foreach($otherSessionValues as $key => $otherValue) {
            $this->sessionManager->set($key, $otherValue);
        }
    }

    /**
     * @param $user
     * @param $organization
     * @param $accessProfile
     * @param $otherSessionValues
     * @param array $loginUserInfo
     * @param $modulesVersionError
     * @param $loginUser
     * @param $loginToken
     * @return array
     */
    private function factorySessionData($user, $organization, $accessProfile, $otherSessionValues, $loginUserInfo,
                                        $modulesVersionError, $loginUser, $loginToken) {
        $this->environment->setLoginTokenInSession($loginUserInfo['TOKEN']);
        if(empty($loginToken)) {
            $loginToken = InstanceProvider::getOAuthCustom()->grantAccessToken(
                $this->libClientId,
                $this->libClientSecret,
                array('sessionId' => InstanceProvider::getSession()->getId()),
                $loginUserInfo['TOKEN']
            );
        }
        $sessionData = array(
            "USER_ID"         => $user['ID'],
            "USER_NAME"       => $user['NAME'],
            "EMAIL"           => $user['EMAIL'],
            "ORGANIZATION_ID" => $user['ORGANIZATION_ID'],

            "TRADING_NAME" => $organization['TRADING_NAME'],
            "COMPANY_NAME" => $organization['COMPANY_NAME'],

            "REDIRECT_URL"     => $loginUserInfo['REDIRECT_URL'],
            "VISIBLE_ELEMENTS" => $loginUserInfo['VISIBLE_ELEMENTS'],
            "EXPIRED_ELEMENTS" => isset($loginUserInfo['EXPIRED_ELEMENTS']) ? $loginUserInfo['EXPIRED_ELEMENTS'] : [],
            
            "IDUTLPERFILFILIAL"   => $this->environment->getIdUtlPerfilFilial(),
            "CDPERFILSELECIONADO" => $this->environment->getNrPerfil(),

            "LOGGED"                => true,
            "SESSION_EXPIRED"       => $this->environment->getSessionExpired(),
            "IS_PENDENT_MIGRATION"  => $this->checkPendentMigrations($user['ORGANIZATION_ID'], $loginUserInfo['REPOSITORIES_INFO']),
            "MODULES_VERSION_ERROR" => $modulesVersionError,
            "ACCESS_PROFILE"        => $accessProfile,

            "LOGIN_TOKEN"           => $loginToken,
            "LOGIN_USER"            => $loginUser
        );
        
        if(isset($loginUserInfo['FILE_SERVER_API_KEY']))
            $sessionData["FILE_SERVER_API_KEY"] = $loginUserInfo["FILE_SERVER_API_KEY"];
        
        foreach($otherSessionValues as $key => $otherValue) {
            $sessionData[$key] = $otherValue;
        }

        return $sessionData;
    }

     /**
     * @param $nrOrg
     * @param $repositoriesInfo
     * @return bool
     */
    private function checkPendentMigrations($nrOrg, $repositoriesInfo) {
        $isPendentMigration = false;

        try {
            $migrationInfo = InstanceProvider::getMigrationService()->getPendentMigrations($nrOrg, $repositoriesInfo);
        } catch (\Exception $e) {
            $migrationInfo = array();
        }

        if(!empty($migrationInfo) && !isset($migrationInfo['ERROR'])) {
            foreach ($migrationInfo as $migration) {
                if ($migration['MIG_LAST_FILE_NUMBER'] && $migration['MIG_LAST_FILE_NUMBER'] < $migration['MIG_HIGHER_FILE_NUMBER']) {
                    $isPendentMigration = true;
                    break;
                }
            }
        }

        return $isPendentMigration;
    }

    /**
     * @param $project
     * @param $eraseHash
     * @param $hash
     */
    public function logout($project, $eraseHash, $hash) {
        $this->sessionManager->start();
        $token = $this->environment->getLoginTokenInSession();
        $user  = $this->environment->getCurrentUserId();
        $dataSet = array (
            "requestType" => "FilterData",
            "filter" => array (
                array (
                    'name' => 'TOKEN',
                    'operator' => '=',
                    'value' => $token
                )
            )
        );
        try {
            $this->utilities->callWebService($token, '/lm_logout', $dataSet, 'logout', $this->loginClientSecret, $this->loginUrl, 'Yes');
        } catch(\Exception $e) {}

        if($eraseHash != 'Y' && $hash) {
            $this->userDataRepository->deleteOperatorHash($user, $project, $hash);
        }

        InstanceProvider::getOAuthCustom()->deleteToken($token);
        $this->sessionManager->destroy();
    }
}