<?php

namespace Ems\BancoTalentos\Util;

use Zeedhi\Framework\Remote\FileUploader\Server;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Utilities {

    public function __construct($zhFileUploader) {
        $this->zhFileUploader = $zhFileUploader;
    }

    private const PRODUCT_ID = '3914';

    public static function getRemoteFileSize($url) {
        $data = get_headers($url, true);
        if (isset($data['Content-Length'])) {
            return (int)$data['Content-Length'];
        }
        return null;
    }

    public static function downloadAndRemoveFile($fileDir, $fileName, $fileType = 'text/plain', $isRemoveFile = true) {
        if(strpos($fileDir, 'http') === false) {
            $fileSize = filesize($fileDir);
        } else {
            $fileSize = self::getRemoteFileSize($fileDir);
            $header   = array (
                "apiKey: " . self::getFileServerApikey()
            );
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $fileDir);
            curl_setopt($ch, CURLOPT_POST, false);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
            curl_setopt($ch, CURLOPT_TIMEOUT, 30);

            $resultado = curl_exec($ch);
            $error = curl_error($ch);
            curl_close($ch);

            if (!empty($error)) {
                throw new \Exception($error);
            } else {
                // $fileDir = /*self::getTempLogPathParameter() .*/ DIRECTORY_SEPARATOR . $fileName;
                $fileDir = '/tmp' . DIRECTORY_SEPARATOR . $fileName;
                var_dump($resultado);die;
                file_put_contents($fileDir, $resultado);
            }
        }

        header('Accept-Ranges: none');
        header("Content-Type: " . $fileType);
        header("Content-Length: " . $fileSize);
        header("Content-Disposition: attachment; filename=" . $fileName);
        readfile($fileDir);

        if($isRemoveFile) {
            unlink($fileDir);
        }
        exit;
    }

    public function saveFileOnFileServer($b64FileContent, $fileExtension, $nrorg, $fileDir = null, $fileName = null, $customFolder='') {
        try {
            if ($fileName === null) {
                $fileName = $this->geraNomeArquivo($fileExtension);
            }

            if ($fileDir === null) {
                $fileDir = $this->geraDiretorioArquivo($fileExtension, $nrorg);
            }

            $fileFullDir = $fileDir . $fileName . "." . $fileExtension;
            $lastModifiedDate = DateUtil::getDataAtual()->format(DateUtil::FORMATO_AMERICANO_DATAHORA);
            $lastModified = 1507122512763;
            $fileSize = strlen($b64FileContent);
            $fileType = "application/" . $fileExtension;
            $webkitRelativePath = "";
            $fileFieldName = "DSPATH";

            $row = array(
                "DSPATH" => array(
                    array(
                        // "name"               => "/Ems/Fotos/20/". $fileName . "." . $fileType,
                        "name"               => $fileFullDir,
                        "lastModified"       => $lastModified,
                        "lastModifiedDate"   => $lastModifiedDate,
                        "webkitRelativePath" => $webkitRelativePath,
                        "size"               => $fileSize,
                        "type"               => $fileType,
                        "b64File"            => $b64FileContent
                    )
                )
            );
            $this->zhFileUploader->uploadFilesInRow($row, $fileFieldName, $customFolder);
            $filePath = $row['DSPATH'][0]['path'];
            return $filePath;
        } catch(\Exception $e) {
            throw new \Exception('Não foi possível salvar a foto. ' . $e->getMessage());
        }
    }

    public function geraNomeArquivo(string $fileType) : string {
        $data = new \DateTime();
        switch ($fileType) {
            case 'jpeg':
                return 'foto_marcacao_' . $data->format('d_m_Y_H_i_s');
        }
    }

    public function geraDiretorioArquivo(string $fileType, int $nrorg) : string {
        switch ($fileType) {
            case 'jpeg': 
                return "/Ems/$nrorg/Fotos";
        }
    }

    public function getFileServerApikey() : string {
        return "ea1999f0a10e053ae87d08e2691f65f4";
    }

    public static function getProductBasePath($isModulePath = false) {
        if(strpos(__DIR__, 'modules') === false || $isModulePath) {
            return explode('backend', __DIR__)[0];
        }
        return explode('modules', __DIR__)[0];
    }

    public static function getTokenLifeTimeParameter() {
        return InstanceManager::getInstance()->getParameter('tokenLifeTime');
    }

    public static function getProductId() {
        return self::PRODUCT_ID;
    }
}
